@@ -144,7 +144,8 @@ function  [ retval, sinterfaces, loaded_jars ]  = chk_spreadsheet_support (path_
     dbug = 0;
   endif
 
-  if (dbug < 0 && octave_config_info ("features").JAVA)
+  HAVE_JAVA = __have_feature__ ("JAVA");
+  if (dbug < 0 && HAVE_JAVA)
     ## Remove loaded Java class libs from the javaclasspath
     if (dbug < -2 && numel (loaded_jars))
       printf ("Removing javaclasspath entries loaded by chk_spreadsheet_support:\n");
@@ -191,7 +192,7 @@ function  [ retval, sinterfaces, loaded_jars ]  = chk_spreadsheet_support (path_
   if (dbug > 1)
     printf ("1. Checking Octave's Java support... ");
   endif
-  if (! octave_config_info.features.JAVA)
+  if (! HAVE_JAVA)
     ## Nothing to do here anymore
     if (abs (dbug) > 1)
       printf ("none.\nThis Octave has no built-in Java support. Skipping Java checks\n");
@@ -256,6 +257,7 @@ function  [ retval, sinterfaces, loaded_jars ]  = chk_spreadsheet_support (path_
     endif
   catch
     ## No Java support
+    tmp1 = 0;
   end_try_catch
   if (! tmp1)
     ## We can return as for below code Java is required.
