@@ -20,12 +20,13 @@
 ## Author: Philip Nienhuis <prnienhuis@users.sf.net>
 ## Created: 2009-12-13
 
-function [ rawarr, ods] = __JOD_spsh2oct__ (ods, wsh, crange, spsh_opts)
+function [ rawarr, ods, rstatus] = __JOD_spsh2oct__ (ods, wsh, crange, spsh_opts)
 
   persistent months;
   months = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN", ...
             "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};
 
+  rstatus = 0;
   ## Check jOpenDocument version
   sh = ods.workbook.getSheet (0);
   cl = sh.getCellAt (0, 0);
@@ -198,5 +199,6 @@ function [ rawarr, ods] = __JOD_spsh2oct__ (ods, wsh, crange, spsh_opts)
 
   ## Keep track of data rectangle limits
   ods.limits = [lcol, rcol; trow, brow];
+  rstatus = 1;
 
 endfunction
