@@ -1,4 +1,4 @@
-## Copyright (C) 2013,2014 Philip Nienhuis
+## Copyright (C) 2013,2015 Philip Nienhuis
 ## Copyright (C) 2013 Markus Bergholz (.xlsx & archive unzip stuff)
 ## 
 ## This program is free software; you can redistribute it and/or modify
@@ -24,29 +24,6 @@
 ## Author: Philip Nienhuis <prnienhuis at users.sf.net>
 ## File open stuff by Markus Bergholz
 ## Created: 2013-09-08
-## Updates:
-## 2013-09-09 Wipe temp dir after opening as all content is in memory
-##            FIXME this needs to be adapted for future OOXML support
-## 2013-09-23 Fix copyright messages
-## 2013-10-02 More comments
-## 2013-10-20 Adapted parts of Markus' .xlxs code
-## 2013-11-10 (MB) Compacted sheet names & rid code in xlsx section
-## 2013-11-12 Rely on private/__unpack (patched unpack.m from core Octave)
-## 2013-11-16 Replace fgetl calls by fread to cope with EOLs
-## 2013-12-13 Fix sheet names parsing regexpr
-## 2013-12-14 Fix sheet names parsing regexpr # 2 (attrib order can change =>
-##            that's why an XML parser is superior over regular expressions)
-## 2013-12-27 Use one variable for processed file type
-##     ''     Shuffled code around to file type order
-## 2014-01-22 Open new files from template directory in io script directory
-## 2014-01-23 Move confirm_recursive_rmdir to __OCT_spsh_close__.m
-## 2014-01-29 Support for xlsx and gnumeric
-## 2014-02-08 Fix wrong function name in error msg (couldn't be unzipped)
-## 2014-03-17 Simplify sheet names discovery
-##     ''     Ignore SheetId (nowhere used)
-## 2014-03-18 Reinstate SheetId (was used somewhere after all)
-## 2014-04-30 Close file handle for gnumeric files
-## 2014-08-25 Precautions to make sure we get back to original work dir
 
 function [ xls, xlssupport, lastintf] = __OCT_spsh_open__ (xls, xwrite, filename, xlssupport, ftype)
 
@@ -73,7 +50,7 @@ function [ xls, xlssupport, lastintf] = __OCT_spsh_open__ (xls, xwrite, filename
   opwd = pwd;
   if (ftype == 5)
     ## Gnumeric xml files are gzipped
-    system (sprintf ("gzip -d -c -S=gnumeric %s > %s", templ, tmpdir));
+    system (sprintf ('gzip -d -c -S=gnumeric "%s" > %s', templ, tmpdir));
     fid = fopen (tmpdir, 'r');
     xml = fread (fid, "char=>char").';
     ## Close file handle, don't delete file
@@ -83,7 +60,7 @@ function [ xls, xlssupport, lastintf] = __OCT_spsh_open__ (xls, xwrite, filename
     try
       unpack (templ, tmpdir, "unzip");
       ## Allow unpack dust to settle down (lazy write file I/O)
-      sleep (0.2);
+      pause (0.2);
     catch
       printf ("file %s couldn't be unpacked. Is it the proper file format?\n", filename);
       xls = [];
@@ -124,6 +101,20 @@ function [ xls, xlssupport, lastintf] = __OCT_spsh_open__ (xls, xwrite, filename
       xls.sheets.sh_names = cell2mat (regexp (sheets, '<sheet name="(.*?)"', "tokens"));
       xls.sheets.rid = str2double (cell2mat (regexp (sheets, ' r:id="rId(\d+)"', "tokens")));
       xls.sheets.sheetid = str2double (cell2mat (regexp (sheets, ' sheetId="(\d+)"', "tokens")));
+      
+      ## read worksheet.xml.rels
+      fid = fopen (sprintf ("%s/xl/_rels/workbook.xml.rels", tmpdir));
+      if (fid > 0)
+        str = fread (fid, "char=>char").';
+        fclose (fid);
+        xls.sheets.rels = ...
+         regexp (str, ['<Relationship Id="rId(\d+)" ' ...
+                       'Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/worksheet" ' ...
+                       'Target="worksheets/sheet(\d+).xml"[^/>]*/>'], "tokens");
+        xls.sheets.rels = reshape (str2double (cell2mat (xls.sheets.rels)), [], columns(xls.sheets.rels))';
+      else
+        xls.sheets.rels = reshape([xls.sheets.rid xls.sheets.rid], [], columns (xls.sheets.rid));
+      endif
 
     endif
 
