@@ -96,11 +96,16 @@ function [ xls, xlssupport, lastintf ] = __UNO_spsh_open__ (xls, xwrite, filenam
     catch
       ## Check if we have a 32 vs. 64 bit clash
       printf ("\nOops, Octave hit an error.\n");
-      if (strcmpi (octave_config_info.USE_64_BIT_IDX_T, "yes") || ...
-          strncmpi (octave_config_info("canonical_host_type"), "x86_64", 6))
+      ## Newer Octave-4.1.0+ has ENABLE_64, older Octave has USE_64_BIT_IDX_T
+      try
+        amd64y = strcmpi (octave_config_info ("USE_64_BIT_IDX_T"), "yes");
+      catch
+        amd64y = octave_config_info.ENABLE_64;
+      end_try_catch
+      if (amd64y || ! isempty (strfind (lower (computer ("arch")), "x86_64")))
         printf ("Maybe you have 32-bit Libre/OpenOffice installed?\n");
         printf ("64-bit Octave requires 64-bit L.O. / OOo.\n\n");
-      elseif (strncmpi (octave_config_info("canonical_host_type"), "i686", 4))
+      elseif (! isempty (strfind (lower (computer ("arch")), "i686")))
         printf ("Maybe you have 64-bit Libre/OpenOffice installed?\n");
         printf ("32-bit Octave requires 32-bit L.O. / OOo.\n\n");
       endif
