@@ -46,7 +46,7 @@ function [odsinterfaces] = getodsinterfaces (odsinterfaces)
   persistent uno_1st_time = 0;
 
   if (isempty (has_java))
-    has_java = octave_config_info.features.JAVA;
+    has_java = __have_feature__ ("JAVA");
   endif
 
   if (isempty (odsinterfaces.OTK) && isempty (odsinterfaces.JOD) ...
