@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010,2011,2012,2013 Philip Nienhuis
+## Copyright (C) 2009-2016 Philip Nienhuis
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -25,33 +25,32 @@
 ## Calling xlsopen without specifying a return argument is fairly useless!
 ##
 ## xlsopen works with interfaces, which are links to external software.
-## For reading from OOXML (Excel 2007 and up), ODS 1.2 and Gnumeric no
-## additional software is required when the OCT interface is used. For all
-## other spreadsheet formats and for writing to spreadsheet files, you need
-## MS-Excel (95 - 2013), or a Java JRE plus Apache POI >= 3.5 and/or JExcelAPI
+## For I/O from/to OOXML (Excel 2007 and up), ODS 1.2 and Gnumeric, no
+## additional software is required when the OCT interface is used (see below).
+## For all other spreadsheet formats, you need one or more of MS-Excel
+## (95 - 2013), or a Java JRE plus Apache POI >= 3.5 and/or JExcelAPI
 ## and/or OpenXLS and/or OpenOffice.org (or clones) installed on your computer
 ## + proper javaclasspath set. These interfaces are referred to as COM, POI,
 ## JXL, OXS, and UNO, resp., and are preferred in that order by default
-## (depending on their presence). The OCT interface has the lowest priority.
-## For OOXML read/write support, in addition to Apache POI support you also
-## need the following jars in your javaclasspath: poi-ooxml-schemas-3.5.jar,
-## xbean.jar and dom4j-1.6.1.jar (or later versions). Later OpenOffice.org
-## versions (UNO interface) have support for OOXML as well.
-## Excel'95 spreadsheets can only be read by JExcelAPI and OpenOffice.org.
-## For just reading OOXML (.xlsx or .xlsm), no Java or add-on packages are 
-## required; but currently you loose a bit of the flexibility of the other
-## interfaces.
+## (depending on their presence). Currently the OCT interface has the lowest
+## priority as it is still experimental.
+## For OOXML read/write support in principle no additional SW is needed.
+## However, the COM, POI and UNO interfaces may provide better OOXML write
+## performance and/or more flexibility.
+## Excel'95 spreadsheets (BIFF5) can only be read using the COM (Excel-ActiveX),
+## JXL (JExcelAPI), and UNO (Open-/LibreOffice) interfaces.
 ##
-## @var{filename} should be a valid .xls or xlsx Excel file name (including
+## @var{filename} should be a valid .xls or .xlsx Excel file name (including
 ## extension). But if you use the COM interface you can specify any extension
 ## that your installed Excel version can read AND write; the same goes for UNO
 ## (OpenOffice.org). Using the other Java interfaces, only .xls or .xlsx are
 ## allowed. If @var{filename} does not contain any directory path, the file
-## is saved in the current directory.
+## is saved in the current directory.  Reading/writing .xlsm and .xlsb files
+## may be possible using the COM and UNO interfaces only, but is untested.
 ##
-## If @var{readwrite} is set to 0 (default value) or omitted, the Excel file
-## is opened for reading. If @var{readwrite} is set to True or 1, an Excel
-## file is opened (or created) for reading & writing.
+## If @var{readwrite} is set to 0 (default value) or omitted, the spreadsheet
+## file is opened for reading. If @var{readwrite} is set to true or 1, a
+## spreadsheet file is opened (or created) for reading & writing.
 ##
 ## Optional input argument @var{reqintf} can be used to override the Excel
 ## interface that otherwise is automatically selected by xlsopen. Currently
@@ -82,64 +81,6 @@
 
 ## Author: Philip Nienhuis <prnienhuis at users.sf.net>
 ## Created: 2009-11-29
-## Updates:
-## 2010-01-03 Added OOXML support
-## 2010-01-10 Changed (java) interface preference order to COM->POI->JXL
-## 2010-01-16 Removed echoeing debug info in POI stanza
-## 2010-03-01 Removed javaclasspath check for rt.jar
-## 2010-03-14 Fixed check on xwrite flag lines 204+, if xlsopen fails xls ptr
-##            should be []
-## 2010-08-25 Improved help text
-## 2010-09-27 Improved POI help message for unrecognized .xls format to hint for BIFF5/JXL
-## 2010-10-20 Improved code for tracking changes to new/existing files
-##     ''     Lots of code cleanup, improved error checking and catching
-##     ''     Implemented fallback to JXL if POI can't read a file.
-## 2010-10-30 More fine-grained file existence/writable checks
-## 2010-11-01 Added <COM>.Application.DisplayAlerts=0 in COM section to avoid Excel pop-ups
-## 2010-11-05 Option for multiple requested interface types (cell array)
-##     ''     Bug fix: JXL fallback from POI for BIFF5 is only useful for reading
-## 2010-11-05 Slight change to reporting to screen
-## 2010-11-08 Tested with POI 3.7 (OK)
-## 2010-11-10 Texinfo header updated
-## 2010-12-01 Small bugfix - reset xlssupport in l. 102
-## 2010-12-06 Textual changes to info header 
-## 2011-03-26 OpenXLS support added
-## 2011-05-18 Experimental UNO support added, incl. creating new spreadsheets
-## 2011-05-22 Textual changes in header
-## 2011-05-29 Cleanup of comments & messages
-## 2011-09-03 Reset chkintf to [] if no xls support was discovered (to allow
-##            rediscovery of interfaces between xlsopen calls, e.g. javaclasspath changes)
-## 2011-09-08 Minor code cleanup
-## 2012-01-26 Fixed "seealso" help string
-## 2012-06-06 Improved interface detection logic. No more messages if same interface is
-##            requested & used consecutively
-## 2012-06-07 Fixed mixed-up lastintf assignments for POI and JXL
-## 2012-09-02 (in UNO section) web adresses need only two consecutive slashes
-## 2012-09-03 (in UNO section) replace canonicalize_file_name on non-Windows to
-##            make_absolute_filename (see bug #36677)
-## 2012-10-07 Moved subfunc getxlsinterfaces to ./private
-##     ''     Moved all interface-specific file open stanzas to separate ./private funcs
-## 2012-10-24 Style fixes
-## 2012-12-18 Improved warning/error messages
-## 2013-02-24 Temporarily disabled OXS and UNO
-## 2013-05-16 Fix fallback to JXL for OOXML files
-## 2013-06-18 Re-enabled OXS and UNO
-## 2013-09-01 Allow input of filename w/o suffix for Excel files
-## 2013-09-02 Better fix for undue fallback to JXL for OOXML files
-##     ''     Fixed wrong error message about OXS and UNO not being supported
-## 2013-09-30 Native Octave interface ("OCT") for reading .xlsx
-##     ''     Adapted header to OCT (also Excel 2013 is supported)
-## 2013-10-01 Some adaptations for gnumeric
-## 2013-10-20 Overhauled file extension detection logic
-## 2013-11-03 Improved interface selection (fix a.o., fallback to JXL for xlsx)
-## 2013-11-04 Catch attempts to write with only OCT interface
-## 2013-12-01 Add support for ODS (Excel 2007+ and OpenOffice.org/LibreOffice support it)
-## 2013-12-27 Use one variable for processed file type
-##     ''     Style fixes
-## 2013-12-28 Allow OOXML support for OpenXLS
-## 2014-01-01 Add .csv to supported file extensions
-##     ''     Add warning that UNO will write ODS f. unsupported file extensions
-##     ''     Copyright string update
 
 function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
 
@@ -157,16 +98,33 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
   xlssupport = 0;
 
   if (nargout < 1)
-      usage ("XLS = xlsopen (Xlfile [, Rw] [, reqintf]). But no return argument specified!"); 
+      error ("xlsopen.m: no return argument specified!\n usage:  XLS = xlsopen (Xlfile [, Rw] [, reqintf])\n"); 
   endif
 
   if (! (islogical (xwrite) || isnumeric (xwrite)))
-      usage ("xlsopen.m: numerical or logical value expected for arg ## 2 (readwrite)")
+      error ("xlsopen.m: numerical or logical value expected for arg ## 2 (readwrite)\n")
+  endif
+
+  if (ischar (filename))
+    [pth, fnam, ext] = fileparts (filename);
+    if (isempty (fnam))
+      error ("xlsopen.m: no filename or empty filename specified");
+    endif
+    if (xwrite && ! isempty (pth))
+      apth = make_absolute_filename (pth);
+      if (exist (apth) != 7)
+        error ("xlsopen.m: cannot write into non-existent directory:\n'%s'\n", ...
+               apth);
+      endif
+    endif
+  else
+    error ("xlsopen.m: filename expected for argument #1");
   endif
 
   if (! isempty (reqinterface))
+    intfmsg = "requested";
     if (! (ischar (reqinterface) || iscell (reqinterface)))
-      usage ("Arg ## 3 (interface) not recognized - character value required"); 
+      error ("Arg ## 3 (interface) not recognized - character value required\n"); 
     endif
     ## Turn arg3 into cell array if needed
     if (! iscell (reqinterface))
@@ -195,7 +153,7 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
         elseif (strcmpi (reqintf, 'OCT'))
           xlsinterfaces.OCT = [];
         else 
-          usage (sprintf (["xlsopen.m: unknown .xls interface \"%s\" requested.\n" 
+          error (sprintf (["xlsopen.m: unknown .xls interface \"%s\" requested.\n" 
                  "Only COM, POI, JXL, OXS, UNO, or OCT) supported\n"], reqinterface{}));
         endif
       endfor
@@ -218,6 +176,8 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
         return
       endif
     endif
+  else
+    intfmsg = "available";
   endif
 
   ## Check if Excel file exists. First check for (supported) file name suffix:
@@ -237,7 +197,7 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
           ftype = 1;
         case {".xlsx", ".xlsm", ".xlsb"}          ## Zipped XML / OOXML. Catches xlsx, xlsb, xlsm
           ftype = 2;
-        case "ods"                                ## ODS 1.2 (Excel 2007+ & OOo/LO can read ODS)
+        case ".ods"                               ## ODS 1.2 (Excel 2007+ & OOo/LO can read ODS)
           ftype = 3;
         case ".gnumeric"                          ## Zipped XML / gnumeric
           ftype = 5;
@@ -256,18 +216,12 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
   
   ## Adapt file open mode for readwrite argument
   if (xwrite)
-    ## Catch attempts to write gnumeric
-    if (ftype == 5)
-      error ("There's only read support for gnumeric files");
-    endif
-    ## Catch attempts to write xlsx if only OCT interface is supported
-    if (xlsintf_cnt == 1 && xlsinterfaces.OCT)
-      error ("Only the OCT interface is present | requested, but that has only read support");
-    endif
     fmode = 'r+b';
     if (! has_suffix)
-      ## Add .xls suffix to filename (all Excel versions can write this)
+      ## Provisionally add .xls suffix to filename (most used format)
       filename = [filename ".xls"];
+      ext = ".xls";
+      ftype = 1
     endif
   else
     fmode = 'rb';
@@ -287,11 +241,11 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
   fid = fopen (filename, fmode);
   if (fid < 0)                      ## File doesn't exist...
     if (! xwrite)                   ## ...which obviously is fatal for reading...
+      ## FIXME process open apps (Excel, LibreOffice, etc) before hard crash
       error ( sprintf ("xlsopen.m: file %s not found\n", filename));
     else                            ## ...but for writing, we need more info:
       fid = fopen (filename, 'rb'); ## Check if it exists at all...
       if (fid < 0)                  ## File didn't exist yet. Simply create it
-        printf ("Creating file %s\n", filename);
         xwrite = 3;
       else                          ## File exists, but isn't writable => Error
         fclose (fid);  ## Do not forget to close the handle neatly
@@ -306,6 +260,15 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
   ## Check for the various Excel interfaces. No problem if they've already
   ## been checked, getxlsinterfaces (far below) just returns immediately then.
   xlsinterfaces = getxlsinterfaces (xlsinterfaces);
+
+  ## If no external interface was detected and no suffix was given, use .xlsx
+  if (! has_suffix && ! (xlsinterfaces.COM + xlsinterfaces.POI + ...
+                         xlsinterfaces.JXL + xlsinterfaces.OXS + ...
+                         xlsinterfaces.UNO))
+    ## Just add 'x' - .xls was already added higher up
+    filename = [filename "x"];
+    ftype = 2;
+  endif
   
   ## Initialize file ptr struct
   xls = struct ("xtype",    'NONE', 
@@ -334,14 +297,14 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
     [ xls, xlssupport, lastintf ] = __JXL_spsh_open__ (xls, xwrite, filename, xlssupport, ftype);
   endif
 
-  if ((! xlssupport) && xlsinterfaces.OXS && ftype <= 2)
+  if ((! xlssupport) && xlsinterfaces.OXS && ftype == 1)
     [ xls, xlssupport, lastintf ] = __OXS_spsh_open__ (xls, xwrite, filename, xlssupport, ftype);
   endif
 
   if ((! xlssupport) && xlsinterfaces.UNO && (ftype != 5))
     ## Warn for LO / OOo stubbornness
     if (ftype == 0 || ftype == 5 || ftype == 6)
-      warning ("UNO interface will write ODS format for unsupported file extensions")
+      warning ("UNO interface will write ODS format for unsupported file extensions\n")
     endif
     [ xls, xlssupport, lastintf ] = __UNO_spsh_open__ (xls, xwrite, filename, xlssupport);
   endif
@@ -363,11 +326,12 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
       if (ftype != 6)
         ## This message is appended after message from getxlsinterfaces()
         printf ("None.\n");
-        warning ("xlsopen.m: no support for spreadsheet I/O");
+        warning ("xlsopen.m: no'%s' spreadsheet I/O support with %s interfaces.\n", ...
+                 ext, intfmsg);
       endif
     else
-      ## No match between filte type & interface found
-      warning ("xlsopen.m: file type not supported by %s %s %s %s %s %s", reqinterface{:});
+      ## No match between file type & interface found
+      warning ("xlsopen.m: file type not supported by %s %s %s %s %s %s\n", reqinterface{:});
     endif
     xls = [];
     ## Reset found interfaces for re-testing in the next call. Add interfaces if needed.
