@@ -51,7 +51,7 @@ function str = xmlwrite (varargin)
 
   ## Check Java support and support files
   if (isempty (java_ok))
-    if (! octave_config_info.features.JAVA)
+    if (! __have_features__ ("JAVA"))
       error ("xmlwrite: Octave was built without Java support, exiting");
     elseif (! usejava ("jvm"))
       error ("xmlwrite: no Java JRE or JDK detected, exiting");
